#!/usr/bin/env python
# -*- coding: utf-8 -*-
# schevo_04_driver.py
# Copyright (C) 2010 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved
#
# <LICENSE=ISC>

#new orm abstraction module
from notmm.dbapi.orm import RelationProxy

#default base class to use for extending with custom 
#authentication methods
from authkit.users import Users

#internal schevo db for authentication and authorization
#using notmm.dbapi.orm:DatabaseProxy

from authkit.users.schevo_driver.model import db

__all__ = ['UserManager']

class UserManager(Users):
    """
    Creates a composite proxy object for the User Entity. 

    >>> manager = UserManager()
    >>> user = manager.objects.get(username="erob")
    >>> user
    <User account: "Etienne Robillard">
    >>> user.username == "erob"
    True
    """

    objects = None

    # Keep this for compatibility with Pylons ;-)
    api_version = 0.4

    def __init__(self, *args, **kwargs):
        # Obtain a default query set for available
        # User entities
        self.objects = self._default_manager = RelationProxy(db.User)

    def user_exists(self, username):
        #look up the user
        user_object = self.objects.get(username=username)
        if user_object is not None:
            return True
        return False

    def user_has_password(self, username, password):
        u = self.objects.get(username=username)
        if u.f.password.compare(password):
            return True
        return False
    
    ### Roles and Groups permissions
    #def role_exists(self, roles):
    #    # return True if the roles exists
    #    return False
